//		Copyright (c) 1999 by
//		Advanced Visual Systems Inc.
//		All Rights Reserved
//	
//	This software comprises unpublished confidential information of
//	Advanced Visual Systems Inc. and may not be used, copied or made
//	available to anyone, except in accordance with the license
//	under which it is furnished.
//	
//      This file is under Perforce control
//      $Id: //depot/express/dbproj/dbil/sdi_connect.hxx#1 $
//

//
// file: sdi_connect.hxx
// author: Patrick Li, Advanced Visual System
// purpose: Define class to establish and maintain a connection
//   to a spatial database.
//
#ifndef SDI_CONNECT_DEFINED
#define SDI_CONNECT_DEFINED 1

#include "sdi_driver.hxx"
#include "sdi_shape.hxx"

// Predefine classes as necesary
//
class SDI_Transaction;

class SDI_Connection_Entry {
public:
   SDI_Connection_Entry( SDI_Connection_Entry *previous, SDI_Connection_Entry *next, SDI_Connection *entry );
   virtual ~SDI_Connection_Entry();
   
   SDI_Connection *GetEntry() { return entry; }
   SDI_Connection_Entry *GetNext() { return next; }
   SDI_Connection_Entry *GetPrevious() { return previous; }
   
private:
   SDI_Connection *entry;
   SDI_Connection_Entry *previous;
   SDI_Connection_Entry *next;
   
   // Create private copy constructor and assignment operator 
   // to invalidate copys and assignments
   //
   SDI_Connection_Entry( const SDI_Connection_Entry & ) {}
   SDI_Connection_Entry &operator=(const SDI_Connection_Entry &);
};

class SDI_Connection_List {
public:
   SDI_Connection_List();
   virtual ~SDI_Connection_List();

   void Add( SDI_Connection *entry );
   void Remove( SDI_Connection *entry );

   void Shutdown();

private:
   SDI_Connection_Entry *head;
   SDI_Connection_Entry *tail;

   // Create private copy constructor and assignment operator 
   // to invalidate copys and assignments
   //
   SDI_Connection_List( const SDI_Connection_List & ) {}
   SDI_Connection_List &operator=(const SDI_Connection_List &);
};

// Define SDI_Connection class
//
class SDI_Connection: public SDI_Error_Tool {
public:
   // Constructor and Destructor
   //
   SDI_Connection();
   virtual ~SDI_Connection();
   
   // Connect and Disconnect from server
   //
   SDI_Status Connect( SDI_Supported_APIs type, char *server, 
      char *instance, char *database, char *user, char *password );
   
   SDI_Status Disconnect();

   // Get pointer to current driver
   //
   SDI_Driver *GetDriver() { return m_driver; }
   static void GetAPIs( int *num_apis, char ***apis, int **ids );
   static void FreeAPIs( int num_apis, char **apis, int *ids );

   // Transaction Processing
   //
   SDI_Status StartTransaction( SDI_Transaction *transaction );
   const SDI_Transaction *GetTransaction() { return current_transaction; }

   SDI_Status CommitTransaction(int implicit=0);
   SDI_Status RollbackTransaction(int implicit=0);
	SDI_Status CancelTransaction(int implicit=0);

   // Insert, Update, and Delete processing
   //
   SDI_Status Insert( char *table, SDI_Shape *shape, 
                      int num_attrs, SDI_Attribute *attrs, 
                      SDI_Transaction *transaction = NULL );

   SDI_Status Insert( char *table, SDI_Shape_Array *shape, 
                      int num_attrs, SDI_Attribute *attrs, int stopOnError,
                      SDI_Transaction *transaction = NULL, SDI_Status_Callback cb = NULL );

   SDI_Status Update( char *table, char *where, char *shape_column,
                      SDI_Shape *shape, int num_attrs, SDI_Attribute *attrs,
                      SDI_Shape *filter_shape, SDI_FILTER_METHODS filter_method,
                      SDI_Transaction *transaction = NULL, SDI_Status_Callback cb = NULL );

   SDI_Status Delete( char *table, char *where, char *shape, char *field, 
                      SDI_Shape *filter_shape, SDI_FILTER_METHODS filter_method,
                      SDI_Transaction *transaction = NULL, SDI_Status_Callback cb = NULL );

   // Misc SQL
   //
   SDI_Status ProcessSQL( char *sql_statement, int use_transaction, 
                          SDI_Transaction *transaction = NULL );

   // Data Management Functions
   //
	virtual SDI_Status GetSysInfo( int option, char * table_name, char ** sql);
   virtual SDI_Status GetUserTables( int permissions, int *num_tables, char ***tables );
   virtual SDI_Status FreeUserTables( int num_tables, char **tables );

   virtual SDI_Status DescribeTable( char *table, int *num_columns, SDI_COLUMN_DEF **column_defs );
   virtual SDI_Status FreeTableDescriptions( int num_columns, SDI_COLUMN_DEF *column_defs );
   
   // Utilities
   //
   int IsConnected() const;
   const SDI_Version *GetVersion() const;
   static void Shutdown() { connectlist.Shutdown(); }

private:
   SDI_Driver *m_driver;
   SDI_Supported_APIs m_type;

   int local_transaction;
   SDI_Transaction *current_transaction;
   static SDI_Connection_List connectlist;

   // Create private copy constructor and assignment operator 
   // to invalidate copys and assignments
   //
   SDI_Connection( const SDI_Connection & ) {}
   SDI_Connection &operator=(const SDI_Connection &) { return *this; }
};
#endif
